<?php

//
// innovaphone fax call PHP class
// copyright (c) innovaphone 2011 - 2016
//
// extends class InnovaphonePbxCall
//

require_once('innovaphone.pbx.call.class.php');

class InnovaphoneFaxCall extends InnovaphonePbxCall {

    protected $guid;
    protected $webdavFileName;
    protected $lastState;
    protected $faxState;
    protected $sendingComplete;
    protected $lastFaxState;
    protected $page;
    protected $pages;
    protected $remoteSubscriberId;
    protected $errorCause;
    protected $errorDescription;

    // class constructor
    public function __construct($guid = false) {
        parent::__construct();
        $this->guid                 = $guid;
        $this->webdavFileName       = "";
        $this->lastState            = 0;
        $this->faxState             = "created";
        $this->lastFaxState         = "created";
        $this->sendingComplete      = true;
        $this->page                 = 0;
        $this->pages                = 0;
        $this->remoteSubscriberId   = false;
        $this->errorCause           = false;
        $this->errorDescription     = false;
    }

    public function update(&$callInfo) {
        $ret = false;
        parent::update($callInfo);
        if(!$this->isActive()) {
            $ret = true;
        }
        if(!$this->guid) {
            $confId = $this->getConfId();
            if($confId) {
                $this->guid = $confId;
                $ret = true;
            }
        }
        if($this->getFaxState() == "created" &&
           (($this->getState() & 0xF) == 0x1 ||
            ($this->getState() & 0xF) == 0x2 ||
            ($this->getState() & 0xF) == 0x3)) 
        {
            if($this->getDirection() === 'inbound') {
                $this->faxState = "waiting";
                if(($this->getState() & 0xF) == 0x1) {
                    $this->sendingComplete = false;
                }
            }
            else {
                $this->faxState = "calling";
            }
            $ret = true;
        }
        else if($this->getFaxState() == "waiting" && ($this->getState() & 0xF) == 0x3 && !$this->sendingComplete) {
            $this->sendingComplete = true;
            $ret = true;
        }
        else if(($this->getState() & 0xF) == 0x4) {
            $this->faxState = "alerting";
            $ret = true;
        }
        else if(($this->getState() & 0xF) == 0x5 &&
                ($this->faxState == "created" || $this->faxState == "waiting" || $this->faxState == "calling" || $this->faxState == "alerting")) {
            $this->faxState = "connecting";
            $ret = true;
        }
        else if(((!$this->isActive() && ($this->getState() & 0xF) == 0x0) || ($this->getState() & 0xF) == 0x6 || ($this->getState() & 0xF) == 0x7) &&
                $this->faxState != "completed" && $this->faxState != "aborted" && $this->faxState != "error") {
            
            if($this->getCauseCode() == 17) {
                $this->errorCause   = "busy";
            }
            else if($this->getCauseCode() == 34 || $this->getCauseCode() == 44) {
                $this->errorCause   = "no channnel available";
            }
            else if($this->getFaxState() == "connecting" && $this->getCauseCode() == 16) {
                $this->errorCause       = "no fax device";
            }
            else {
                $this->errorCause       = "call disconnected";
                $this->errorDescription = "release code " . $this->getCauseCode();
            }
            $this->faxState     = "aborted";
            $ret = true;
        }
        foreach($callInfo->info as &$info) {
            if($info->type == 'uui') {
                $this->updateUUI($info->vals);
                $ret = true;
            }
        }
        return $ret;
    }
    
    public function getGuid() {
        return $this->guid;   
    }

    public function getConfId() {
        foreach($this->info as &$info) {
            if($info->type == 'conf') {
                return $info->vals . ($this->getDirection() == "inbound" ? "-I" : "-O");
            }
        }
        return 0;
    }

    public function setAssigned($webdavFileName) {
        $this->webdavFileName   = $webdavFileName;
        $this->faxState         = 'assigned';
    }

    public function setWebdavFileName($webdavFileName) {
        $this->webdavFileName   = $webdavFileName;
    }

    public function getWebdavFileName() {
        return $this->webdavFileName;
    }

    public function getFaxState() {
        return $this->faxState;
    }

    public function getLastState() {
        return $this->lastState;
    }

    public function setLastState($lastState) {
        $this->lastState = $lastState;
    }

    public function getLastFaxState() {
        return $this->lastFaxState;
    }

    public function setLastFaxState($lastFaxState) {
        $this->lastFaxState = $lastFaxState;
    }

    public function getSendingComplete() {
        return $this->sendingComplete;
    }
    
    public function getPage() {
        return $this->page;
    }
    
    public function getPages() {
        return $this->pages;
    }
    
    public function getRemoteSubscriberId() {
        return $this->remoteSubscriberId;
    }

    public function getErrorCause() {
        return $this->errorCause;
    }

    public function getErrorDescription() {
        return $this->errorDescription;
    }

    public function show() {
        parent::show();
        print "Call handle: #" . $this->getCallHandle() . ", Fax state: " . $this->getFaxState() . ", GUID: " . $this->getGuid();
        if($this->getFaxState() == "aborted" || $this->getFaxState() == "error") {
            print ", Cause: " . $this->errorCause . ", Description: " . $this->errorDescription;
        }
        print "\n";
    }

    private function updateUUI($uui) {
        $parsed = parse_url($uui);
        $command = "";
        $args = array();
        if(array_key_exists('path', $parsed)) {
            $command = $parsed['path'];
            if(array_key_exists('query', $parsed)) {
                $arglist = explode('&', $parsed['query']);
                foreach($arglist as $arg) {
                    list($option, $value) = explode('=', $arg, 2);
                    if($option) {
                        $args[$option] = $value;
                    }
                }
            }

            //print "UUI command: " . $command . "\n";
            foreach($args as $arg => $value) {
                //print $arg . " = " . $value . "\n";
            }

            switch($command) {
                case('connected'):
                    $this->faxState = $command;
                    if(array_key_exists('remote-subscriber-id', $args)) {
                        $this->remoteSubscriberId = $args['remote-subscriber-id'];
                    }
                    break;

                case('transmitting'):
                case('receiving'):
                    $this->faxState = $command;
                    if(array_key_exists('page', $args)) {
                        $this->page = $args['page'];
                    }
                    else {
                        $this->page = 0;
                    }
                    if(array_key_exists('pages', $args)) {
                        $this->pages = $args['pages'];
                    }
                    else {
                        $this->pages = 0;
                    }
                    break;

                case('completed'):
                    if($this->faxState != 'aborted' && $this->faxState != 'error') {
                        $this->faxState = $command;
                    }
                    $this->page = 0;
                    if(array_key_exists('pages', $args)) {
                        $this->pages = $args['pages'];
                    }
                    else {
                        $this->pages = 0;
                    }
                    break;

                case('aborted'):
                    $this->faxState     = $command;
                    if(array_key_exists('cause', $args)) {
                        $this->errorCause   = $args['cause'];
                    }
                    if(array_key_exists('description', $args)) {
                        $this->errorDescription   = $args['description'];
                    }
                    break;

                case('error'):
                    $this->faxState = $command;
                    if(array_key_exists('cause', $args)) {
                        $this->errorCause   = $args['cause'];
                    }
                    if(array_key_exists('description', $args)) {
                        $this->errorDescription   = $args['description'];
                    }
                    break;
            }
        }
    }
}

?>